#https://developer.aitrios.sony-semicon.com/edge-ai-sensing/documents/pytorch-model-deployment-guide?version=2025-04-23&progLang=#_Prepare_training_images_and_transfer_learning
import datetime
import numpy as np
import torch
import torch.nn as nn
import torch.optim as optim
import numpy as np
from torchvision import datasets, models, transforms

import time
import os
import argparse


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--model_path', type=str, default='./weight.pth', help='The path to the pytorch model to save')
#    parser.add_argument('--dataset_path', type=str, default='./dataset', help='The path to the data set') # takuya
    parser.add_argument('--dataset_path', type=str, default='./', help='The path to the data set') # takuya
    parser.add_argument('--batch_size', type=int, default=32, help='Batch size')
    parser.add_argument('--epochs', type=int, default=10, help='Number of epoch')
    args = parser.parse_args()

    #<1> Load dataset and normalize the datasets.
    data_transform = transforms.Compose([
            transforms.Resize(size=(224,224)),
            transforms.ToTensor(),
        ])

    image_dataset = datasets.ImageFolder(os.path.join(args.dataset_path, 'train'),
                                              data_transform)

    dataloader = torch.utils.data.DataLoader(image_dataset, batch_size=args.batch_size,
                                                 shuffle=True, num_workers=4)

    dataset_sizes = len(image_dataset)
    class_names = image_dataset.classes
    num_classes = len(image_dataset.classes)  # takuya
    print("num_classes:",num_classes)         # takuya

    device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")

    #<2> Create the model.
    model = models.mobilenet_v2(weights='IMAGENET1K_V1')
#    model.classifier[1] = nn.Linear(in_features=1280, out_features=2) # takuya
    model.classifier[1] = nn.Linear(in_features=1280, out_features=num_classes) # takuya
    model.classifier = nn.Sequential(
                        model.classifier[0],
                        model.classifier[1],
                        nn.Softmax(dim=1)
                    )
    params_to_update = []
    update_param_names = ['classifier.1.weight', 'classifier.1.bias']

    model = model.to(device)
    criterion = nn.CrossEntropyLoss().to(device)

    #<3> Set the layers to be trained for transfer learning.
    for name, param in model.named_parameters():
        if name in update_param_names:
            param.requires_grad = True
            params_to_update.append(param)
        else:
            param.requires_grad = False

    #<4> Train the model.
    optimizer = optim.Adam(params_to_update)

    for epoch in range(args.epochs):
        print('Epoch {}/{}'.format(epoch + 1, args.epochs))
        print('-------------')

        start_time = time.time()

        model.train()

        epoch_loss = 0.0
        epoch_corrects = 0
        batch_num = 0

        for inputs, labels in dataloader:
            inputs = inputs.to(device)
            labels = labels.to(device)

            optimizer.zero_grad()
            outputs = model(inputs)
            loss = criterion(outputs, labels)

            _, preds = torch.max(outputs, 1)

            loss.backward()
            optimizer.step()

            epoch_loss += loss.item() * inputs.size(0)
            epoch_corrects += torch.sum(preds == labels.data)

            print("\r",batch_num,end="")
            batch_num = batch_num + 1

        epoch_loss = epoch_loss / len(dataloader.dataset)
        epoch_acc = epoch_corrects.double() / len(dataloader.dataset)

        total_time = time.time() - start_time
        total_time_str = str(datetime.timedelta(seconds=int(total_time)))
        print(' : Calculation time {}'.format(total_time_str))
        print('{} Loss: {:.4f} Acc: {:.4f}'.format('train', epoch_loss, epoch_acc))

    #<5> Save the trained model.
    torch.save(model.state_dict(), args.model_path)